# Before using this script, you must open "Read_Fluidigm_Functions.R" and run the whole script.
# This script will rely on those functions being in memory.

# Set your working directory (YOU WILL NEED TO CHANGE THIS DEPENDING ON WHERE YOU PUT THE DIRECTORY)
   setwd("~jmgray/Documents/TEACHING/Genetics_bootcamp/")

# Define primer sets for different chips (runs)
   IEGs_c1 <- c("Fosb","Npas4","Nr4a1","Junb","Fos","Egr2","Egr4","Pcsk1","Egr1","Gadd45b","Maff","Ier2","Klf4","Vgf","Arc","Atf7ip2","Nr4a2","Egr3")
   LRGs_c1 <- c("Col10a1","Gprc5a","Bves","Pax1","Bdnf","Pdlim3","Acan","Npffr2","Crh","Rasd1","Serpinb2","Ankrd56","Corin","Pthlh","Fosl2")
   CNTLs_c1 <- c("Casp6","Prkcn","Ssna1","Hs2st1","Mrpl50","Nnmt","Mllt11","Eif4h","Atp2c1","Tubb3","Actb","Gapdh")
   primerSet_c1 <- c(IEGs_c1,LRGs_c1,CNTLs_c1)  
   IEGs_chips2to4 <- c("Fosb","Npas4","Nr4a1","Junb","Fos","Egr2","Egr4","Pcsk1","Egr1","Gadd45b","Maff","Ier2","Klf4","Vgf","Arc","Atf7ip2","Nr4a2","Egr3")
   LRGs_chips2to4 <- c("Col10a1","Gprc5a","Bves","Pax1","Bdnf","Pdlim3","Acan","Npffr2","Crh","Rasd1","Serpinb2","Ankrd56","Corin","Pthlh","Fosl2","Fosl1")
   CNTLs_chips2to4 <- c("Casp6","Prkcn","Ssna1","Hs2st1","Mrpl50","Nnmt","Mllt11","AI597468","Eif4h","Atp2c1","Tubb3","Actb","Gapdh")
   primerSet_c2to4 <- c(IEGs_chips2to4,LRGs_chips2to4,CNTLs_chips2to4) 
   
# Read in raw Fluidigm qPCR data and process into a concentration for each gene in each sample
   # c1 (chip1) is a 96x96 chip with 48 primers (each replicated twice) and includes timecourses from 0-6 hours from 1 and 5 minute pulses of KCl
   c1 <- read_results(fN="Fluidigm/Fluidigm_Chip1_01_01_2011.tsv",IEGs=IEGs_c1,LRGs=LRGs_c1,CNTLs=CNTLs_c1)
   # c2 is a 96x96 chip with 48 primers (each replicated twice) and includes a control NaCl timecourse, as well as a two pulse experiment
   # c2 <- read_results(fN="Fluidigm/Fluidigm_Chip2_03_07_2011.tsv",IEGs=IEGs_chips2to4,LRGs=LRGs_chips2to4,CNTLs=CNTLs_chips2to4)
   # c3 is half of a 96x96 chip with 48 primers (each replicated twice) and 48 samples.  It includes two 6 hour timecourses of sustained KCl, and CHX/ANS treatment
   c3 <- read_results(fN="Fluidigm/Fluidigm_Chip3_04_12_2011.tsv",IEGs=IEGs_chips2to4,LRGs=LRGs_chips2to4,CNTLs=CNTLs_chips2to4)
   
# Plot a standard curve 
   white() # reset the plot window format
   plotStdCurve("Egr1",c1)


# define which sample names correspond to which timepoints and define control signals
   # Translate sample labels into KCl timepoints
   kcl_0136_c1b1 <- data.frame(names=c("Athar 1","Athar 2","Athar 3","Athar 4"),timepoints=c(0,60,180,360))   
   kcl_0136_c1b2 <- data.frame(names=c("Athar 9","Athar 10","Athar 11","Athar 12"),timepoints=c(0,60,180,360))  
   # For each timepoint, calculate a mean control gene average
   (CNTLs_kcl_0136_c1b1 <- plotCNTLs(c1,CNTLs_c1, kcl_0136_c1b1))
   (CNTLs_kcl_0136_c1b2 <- plotCNTLs(c1,CNTLs_c1, kcl_0136_c1b2))

# Plot for any gene a timecourse plot of fold induction
   plotPulse(c1,primerName="Egr1", kcl_0136_c1b1,c= CNTLs_kcl_0136_c1b1)  
   
   

# Plot two biological replicates of sustained KCl, 0,1,6 hours treatment
   plotAllPrimers(x=c1,expt= kcl_0136_c1b1,exptCntl= CNTLs_kcl_0136_c1b1,primers=primerSet_c1,fN="plots/Fluidigm_KCl_b1",newWindow=TRUE)
   plotAllPrimers(x=c1,expt= kcl_0136_c1b2,exptCntl= CNTLs_kcl_0136_c1b2,primers=primerSet_c1,fN="plots/Fluidigm_KCl_b2",newWindow=TRUE)
      
   
  
# Define the experiments and controls for the CHX/ANS experiment
# This just maps sample names onto timepoints, assigning different colors for the drug treatments
  (sustained_kcl_chx_ans_b1 <- data.frame(names=c("Sustained KCl 0 032411","Sustained KCl 20 032411","Sustained KCl 40 032411","Sustained KCl 60 032411","Sustained KCl 120 032411","Sustained KCl 240 032411","Sustained KCl 360 032411","Sustained KCl+CHX 0 032411","Sustained KCl+CHX 20 032411","Sustained KCl+CHX 40 032411","Sustained KCl+CHX 60 032411","Sustained KCl+CHX 120 032411","Sustained KCl+CHX 240 032411","Sustained KCl+CHX 360 032411","Sustained KCl+ANS 0 032411","Sustained KCl+ANS 20 032411","Sustained KCl+ANS 40 032411","Sustained KCl+ANS 60 032411","Sustained KCl+ANS 120 032411","Sustained KCl+ANS 240 032411","Sustained KCl+ANS 360 032411"),timepoints=c(0,20,40,60,120,240,360,0,20,40,60,120,240,360,0,20,40,60,120,240,360),colors=c(rep("black",times=7),rep("red",times=7),rep("green",times=7)),labels=c(rep("no drug",times=7),rep("CHX",times=7),rep("ANS",times=7))))
  (sustained_kcl_chx_ans_b2 <- data.frame(names=c("Sustained KCl 0 033111","Sustained KCl 20 033111","Sustained KCl 40 033111","Sustained KCl 60 033111","Sustained KCl 120 033111","Sustained KCl 240 033111","Sustained KCl 360 033111","Sustained KCl+CHX 0 033111","Sustained KCl+CHX 20 033111","Sustained KCl+CHX 40 033111","Sustained KCl+CHX 60 033111","Sustained KCl+CHX 120 033111","Sustained KCl+CHX 240 033111","Sustained KCl+CHX 360 033111","Sustained KCl+ANS 0 033111","Sustained KCl+ANS 20 033111","Sustained KCl+ANS 40 033111","Sustained KCl+ANS 60 033111","Sustained KCl+ANS 120 033111","Sustained KCl+ANS 240 033111","Sustained KCl+ANS 360 033111"),timepoints=c(0,20,40,60,120,240,360,0,20,40,60,120,240,360,0,20,40,60,120,240,360),colors=c(rep("black",times=7),rep("red",times=7),rep("green",times=7)),labels=c(rep("no drug",times=7),rep("CHX",times=7),rep("ANS",times=7))))   
  (zero_hrs_b1 <- data.frame(names=c("Sustained KCl 0 032411","Sustained KCl+CHX 0 032411","Sustained KCl+ANS 0 032411"),timepoints=c(0,0,0),colors=c("black","red","green"),labels=c("no drug b1","chx b1","ans b1")))
  (zero_hrs_b2 <- data.frame(names=c("Sustained KCl 0 033111","Sustained KCl+CHX 0 033111","Sustained KCl+ANS 0 033111"),timepoints=c(0,0,0),colors=c("black","red","green"),labels=c("no drug b2","chx b2","ans b2")))
   #average the control genes for the CHX/ANS experiment
   cntls_sustained_kcl_chx_ans_b1_14 <- plotCNTLsMulti(c3,CNTLs_chips2to4,sustained_kcl_chx_ans_b1)
   cntls_sustained_kcl_chx_ans_b2_14 <- plotCNTLsMulti(c3,CNTLs_chips2to4,sustained_kcl_chx_ans_b2)
   cntls_zero_hrs_b1_14 <- plotCNTLsMulti(c3,CNTLs_chips2to4,zero_hrs_b1)
   cntls_zero_hrs_b2_14 <- plotCNTLsMulti(c3,CNTLs_chips2to4,zero_hrs_b2)



 # Plot any single gene for the CHX/ANS experiment
   plotPulseMultiColor(c3,primerName="Fos", sustained_kcl_chx_ans_b1,c=cntls_sustained_kcl_chx_ans_b1_14,singularNormalization=T)     



# Plot all genes for the CHX/ANS experiment
   plotAllPrimersMulti(x=c3,expt=sustained_kcl_chx_ans_b1,exptCntl=cntls_sustained_kcl_chx_ans_b1_14,primers=primerSet_c1,fN="Plots/Fluidigm_sustained_kcl_chx_ans_b1",newWindow=TRUE,singularNormalization=T)
   plotAllPrimersMulti(x=c3,expt=sustained_kcl_chx_ans_b2,exptCntl=cntls_sustained_kcl_chx_ans_b2_14,primers=primerSet_c1,fN="Plots/Fluidigm_sustained_kcl_chx_ans_b2",newWindow=F,singularNormalization=T)   



   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   