# some generic stuff

options(stringsAsFactors = F)

black <- function(){
	par(bg="black",fg="white",col.lab="white",col.axis="white",col.main="white", col.sub="white")
	par(mar=c(5.2,5.8,4,2)+0.1,cex=1,cex.main=2,cex.lab=2,cex.axis=1.5)
	#par(xaxs='i',yaxs='i')
}
white <- function(){
	par(bg="white",fg="black",col.lab="black",col.axis="black",col.main="black", col.sub="black")
	par(mar=c(5.2,5.8,4,2)+0.1,cex=1,cex.main=2,cex.lab=2,cex.axis=1.5,mfrow=c(1,1),pch=19)
	#par(xaxs='i',yaxs='i')
}
white()

# read in results from a specified file (with extra headers removed)
read_results <- function(fN,IEGs,LRGs,CNTLs){ 
	
   # troubleshooting
   # fN <- "/Volumes/Neurobio/GreenbergLab/Joe J Ling/IEG_project/Fluidigm/results/1361523246_2011_1_11_Taqman_Fluidigm_Assays_01142011_for_R.tsv"
   # fN <- "/Volumes/Neurobio/GreenbergLab/Joe J Ling/IEG_project/Fluidigm/results/1361523246_2011_1_11_Taqman_Fluidigm_Assays_01142011_for_R.tsv"

   head(x <- read.delim(file=fN,as.is=TRUE))

   x$sampleNum <- sapply(x$ID,function(x){strsplit(x,split="-")[[1]][1]})
   x$assayNum <- sapply(x$ID,function(x){strsplit(x,split="-")[[1]][2]})

   label_by_class <- function(myNames,IEGs,LRGs,CNTLs){
	 output <- myNames
	 output[which(!is.na(match(myNames,IEGs)))] <- "IEG"
	 output[which(!is.na(match(myNames,LRGs)))] <- "LRG"
	 output[which(!is.na(match(myNames,CNTLs)))] <- "CNTL"
	 output
   }
   head(x$gene_class <- label_by_class(x$Name.1,IEGs,LRGs,CNTLs))

   # reorder sensibly
   head(x <- x[order(x$assayNum,x$sampleNum),])

   # rename some columns sensibly
   colnames(x)[which(colnames(x) == "Name.1")] <- "primer"
   colnames(x)[which(colnames(x) == "Name")] <- "sampleName"
   
   # annotate the standard curve
   x$Conc <- 0.05
   x$Conc[which(x$sampleName == "1/6hr kcl 1")] <- 1
   x$Conc[which(x$sampleName == "1/6hr kcl 1:10")] <- 0.1
   x$Conc[which(x$sampleName == "1/6hr kcl 1:100")] <- 0.01
   x$Conc[which(x$sampleName == "1/6hr kcl 1:1000")] <- 0.001
   x$Conc[which(x$sampleName == "1/6hr kcl 1:10000")] <- 0.0001
   
   # label the standards
   x$SampleClass <- NA
   x$SampleClass[grep("1/6hr kcl 1",x$sampleName)] <- "standard"
   
   # annotate biological and technical replicates
   x$sampleTechRep <- NA
   x$sampleTechRep[which(!is.na(match(x$sampleNum,c("S01","S13","S25","S37","S49"))))] <- 1 # this is sample technical replicates (look them up)
   x$sampleTechRep[which(!is.na(match(x$sampleNum,c("S07","S19","S31","S43","S55"))))] <- 2 # there could also be assay technical reps, ignored here
   
   primer_names <- unique(x$primer)
   primer_names <- primer_names[which(primer_names!="water")] # if there isn't at least two points to draw a line, it results in "error in int_abline..."
   head(std_curves <- aperm(sapply(primer_names,plotStdCurve,x)))

   # add a concentration column based on standard curve
   head(x$realConc <- apply(x,1,getConc,std_curves))
   x$realConc <- as.numeric(x$realConc)

   # return the data
   x
}



# plot replicability across all primers and samples on a chip
# the Quad version assumes there are at least some 2x sample and 2x assay replicates 
plotReplicabilityQuad <- function(x,myWhich="both"){
#x <- p1   	
      a <- x$Value 
      b <- x$sampleNum
      c <- x$assayNum
      d <- x$primer
      e <- x$sampleName

      head(y <- data.frame(Value=a,sampleNum=b,assayNum=c,primer=d,sampleName=e))
      head(z <- y[order(y$sampleName,y$primer,y$assayNum),],n=20)
      z$Value[which(z$Value == 999)] <- NA
      
      
      # remove any samples /assays for which we don't have quadruplicates
      primer_sample <- paste(z$primer,z$sampleName)
      unique_primer_sample <- unique(primer_sample)
      test <- function(id,x){
         sum(!is.na(match(x,id)))	
      }
      passing <- unique_primer_sample[which(sapply(unique_primer_sample,test,primer_sample) == 4)]
      dim(z <- z[which(!is.na(match(primer_sample,passing))),])
      
      # this is really a BS way of doing this that may not work on all chips; assumes quadruplicate 2x sample 2x assay
      head(S1A1 <- z[seq(from=1,to=length(z$Value),by=4),]$Value)
      head(S2A1 <- z[seq(from=2,to=length(z$Value),by=4),]$Value)
      head(S1A2 <- z[seq(from=3,to=length(z$Value),by=4),]$Value)
      head(S2A2 <- z[seq(from=4,to=length(z$Value),by=4),]$Value)
      myMin <- min(z$Value,na.rm=T)
      myMax <- max(z$Value,na.rm=T)
      if(myWhich == "both"){
         plot(c(S1A1,S2A1),c(S1A2,S2A2),main="Fluidigm replicability",frame.plot=F,xlab="replicate 1",ylab="replicate 2",xlim=c(myMin,myMax),ylim=c(myMin,myMax),cex=0.1)     
         points(c(S1A1,S1A2),c(S2A1,S2A2),col="red",cex=0.1) 
         legend("topleft",legend=c("assay reps","sample reps"),text.col=c("black","red"),bty="n",cex=2)
      } else if(myWhich == "sample") {
      	 plot(c(S1A1,S1A2),c(S2A1,S2A2),main="Sample replicability",frame.plot=F,xlab="replicate 1",ylab="replicate 2",xlim=c(myMin,myMax),ylim=c(myMin,myMax),cex=0.1)
      } else if (myWhich == "assay"){
         plot(c(S1A1,S2A1),c(S1A2,S2A2),main="Assay replicability",frame.plot=F,xlab="replicate 1",ylab="replicate 2",xlim=c(myMin,myMax),ylim=c(myMin,myMax),cex=0.1)
      }
}







# plot replicability for a specific sample
plotReplicabilityBySample <- function(sampleName,x){

   	
      # look at replicability
      a <- x[which(x$sampleName==sampleName),]$Value 
      b <- x[which(x$sampleName==sampleName),]$sampleNum
      c <- x[which(x$sampleName==sampleName),]$assayNum
      d <- x[which(x$sampleName==sampleName),]$primer

      head(y <- data.frame(Value=a,sampleNum=b,assayNum=c,primer=d))
      head(z <- y[order(y$primer,y$assayNum),],n=20)
      z$Value[which(z$Value == 999)] <- NA
      
      head(S1A1 <- z[seq(from=1,to=length(z$Value),by=4),]$Value)
      head(S2A1 <- z[seq(from=2,to=length(z$Value),by=4),]$Value)
      head(S1A2 <- z[seq(from=3,to=length(z$Value),by=4),]$Value)
      head(S2A2 <- z[seq(from=4,to=length(z$Value),by=4),]$Value)
      myMin <- min(z$Value,na.rm=T)
      myMax <- max(z$Value,na.rm=T)
      plot(c(S1A1,S2A1),c(S1A2,S2A2),main="Fluidigm replicability",frame.plot=F,xlab="replicate 1",ylab="replicate 2",xlim=c(myMin,myMax),ylim=c(myMin,myMax))     
      points(c(S1A1,S1A2),c(S2A1,S2A2),col="red") 
      legend("topleft",legend=c("assay reps","sample reps"),text.col=c("black","red"),bty="n",cex=2)
        
	  assayR2 <- cor.test(c(S1A1,S2A1),c(S1A2,S2A2))$estimate
	  sampleR2 <- cor.test(c(S1A1,S1A2),c(S2A1,S2A2))$estimate
	  mean(assayR2,sampleR2)
}

# plot replicability for a specific primer
plotReplicabilityByPrimer <- function(primer,x,removeRT=T){


#primer <- "Fos"
#x <- p2

   	  if(removeRT == TRUE){x <- x[grep("RT-",x$sampleName,invert=T),]} # remove RT- samples
   	
      # look at replicability
      a <- x[which(x$primer==primer),]$Value 
      b <- x[which(x$primer==primer),]$sampleNum
      c <- x[which(x$primer==primer),]$assayNum
      d <- x[which(x$primer==primer),]$sampleName

      head(y <- data.frame(Value=a,sampleNum=b,assayNum=c,sample=d))
      head(z <- y[order(y$sample,y$assayNum),],n=20)
      z$Value[which(z$Value == 999)] <- NA
     
      head(S1A1 <- z[seq(from=1,to=length(z$Value),by=4),]$Value)
      head(S2A1 <- z[seq(from=2,to=length(z$Value),by=4),]$Value)
      head(S1A2 <- z[seq(from=3,to=length(z$Value),by=4),]$Value)
      head(S2A2 <- z[seq(from=4,to=length(z$Value),by=4),]$Value)

      myMin <- min(z$Value,na.rm=T)
      myMax <- max(z$Value,na.rm=T)
      plot(c(S1A1,S2A1),c(S1A2,S2A2),main=paste("Fluidigm replicability (",primer,")",sep=""),frame.plot=F,xlab="replicate 1",ylab="replicate 2",xlim=c(myMin,myMax),ylim=c(myMin,myMax))     
      points(c(S1A1,S1A2),c(S2A1,S2A2),col="red") 
      legend("topleft",legend=c("assay reps","sample reps"),text.col=c("black","red"),bty="n",cex=2)  

	  assayR2 <- cor.test(c(S1A1,S2A1),c(S1A2,S2A2))$estimate
	  sampleR2 <- cor.test(c(S1A1,S1A2),c(S2A1,S2A2))$estimate
	  mean(assayR2,sampleR2)
	  # to spit out the table: z
}


# for any given primer just plot the std curves by replicate
plotStdCurveByRep <- function(primer,x,printData=F){
   	  # sample replicate 1
      toPl <- x[which(x$primer == primer & x$SampleClass == "standard" & x$sampleTechRep == 1 & x$Value != 999 & x$Call == "Pass"),]
      if(dim(toPl)[1] > 1){
         plot(x=log10(toPl$Conc),y=toPl$Value,ylim=c(0,40),frame.plot=F,main=primer,xlab="standard dilution",ylab="Ct")
         myX <- log10(toPl$Conc); myY <- toPl$Value
         abline(myLM <- lm(myY ~ myX))
         r2.1 <- summary(myLM)$r.squared
         myRange.1 <- range(myY) 
         slope1 <- myLM$coefficients[2]
         int1 <- myLM$coefficients[1]
      } 
      
      # sample replicate 2
      toPl <- x[which(x$primer == primer & x$SampleClass == "standard" & x$sampleTechRep == 2 & x$Value != 999 & x$Call == "Pass"),]
      if(dim(toPl)[1] > 1){
         points(x=log10(toPl$Conc),y=toPl$Value,ylim=c(0,40),col="red")
         myX <- log10(toPl$Conc); myY <- toPl$Value
         abline(myLM <- lm(myY ~ myX),col="red")       
      	 r2.2 <- summary(myLM)$r.squared
         myRange.2 <- range(myY) 
         slope2 <- myLM$coefficients[2]
         int2 <- myLM$coefficients[1]
      } 
}
      

# for any given primer plot the standard curve values
# and spit out enough info to evaluate the standard curve later  
   plotStdCurve <- function(primer,x,printData=F){      

   	# aggregated analysis of both replicates
    toPl <- x[which(x$primer == primer & x$SampleClass == "standard" & x$Value != 999),]

    if(length(unique(toPl$sampleName)) > 1){
      if(dim(toPl)[1] > 1){
         plot(x=log10(toPl$Conc),y=toPl$Value,ylim=c(0,40),frame.plot=F,main=primer,xlab="standard dilution",ylab="Ct")
         myX <- log10(toPl$Conc); myY <- toPl$Value
         abline(myLM <- lm(myY ~ myX))
         r2.a <- summary(myLM)$r.squared
         myRange.a <- range(myY) 
         slopea <- myLM$coefficients[2]
         inta <- myLM$coefficients[1]
      } else {
      	 r2.a <- NA; myRange.a <- NA; slopea <- NA; inta <- NA
      }
      
      
      # processing both/either replicate
      # spit out a list with the results of the standard curve analysis
      if(printData == F){
      	list(r2=r2.a,meanL=myRange.a[1],meanH=myRange.a[2],slope=slopea,int=inta)
      } else {
      	x[which(x$primer == primer & x$SampleClass == "standard" & x$Value != 999 & x$Call == "Pass"),]
      }
   } else {
      if(printData == F){
      	list(r2=NA,meanL=NA,meanH=NA,slope=NA,int=NA)
      } else {
      	x[which(x$primer == primer & x$SampleClass == "standard" & x$Value != 999 & x$Call == "Pass"),]
      }   	        		
   }
  }
   
#plotStdCurve("Fos",x)
   


   # this is a defunct version (i think...)
   plotStdCurve_defunct <- function(primer,x,printData=F){      
   	  # aggregated analysis of both replicates
      toPl <- x[which(x$primer == primer & x$SampleClass == "standard" & x$Value != 999 & x$Call == "Pass"),]
      if(dim(toPl)[1] > 1){
         plot(x=log10(toPl$Conc),y=toPl$Value,ylim=c(0,40),frame.plot=F,main=primer,xlab="standard dilution",ylab="Ct")
         myX <- log10(toPl$Conc); myY <- toPl$Value
         abline(myLM <- lm(myY ~ myX))
         r2.a <- summary(myLM)$r.squared
         myRange.a <- range(myY) 
         slopea <- myLM$coefficients[2]
         inta <- myLM$coefficients[1]
      } else {
      	 r2.a <- NA; myRange.a <- NA; slopea <- NA; inta <- NA
      }
      

      # processing both/either replicate
      if(any(!is.na(c(myRange.1[1],myRange.2[1])))){
      	 meanL <- mean(myRange.1[1],myRange.2[1],na.rm=T)
      } else {meanL <- NA}	  
      if(any(!is.na(c(myRange.1[2],myRange.2[2])))){
      	 meanH <- mean(myRange.1[2],myRange.2[2],na.rm=T)
      } else {meanH <- NA}
      # spit out a list with the results of the standard curve analysis
      if(printData == F){
      	list(r2=r2.a,r2.1=r2.1,r2.2=r2.2,meanL=meanL,meanH=meanH,slope=slopea,slope1=slope1,slope2=slope2,int=inta,int1=int1,int2=int2)
      } else {
      	x[which(x$primer == primer & x$SampleClass == "standard" & x$Value != 999 & x$Call == "Pass"),]
      }
   }   
   

# for each sample, use the corresponding appropriate standard curve to get a concentration value
   getConc <- function(x,std_curves){
   	    # label the data
   		Ct <- as.numeric(x[7])
   		primerName <- x[5]
   		
   		# label/extract the std curve info
		minRange <- std_curves[which(rownames(std_curves) == primerName),]$meanL
		maxRange <- std_curves[which(rownames(std_curves) == primerName),]$meanH
		r2 <- std_curves[which(rownames(std_curves) == primerName),]$r2
		slope <- as.numeric(std_curves[which(rownames(std_curves) == primerName),]$slope)
		int <- as.numeric(std_curves[which(rownames(std_curves) == primerName),]$int)

		if(all(!is.na(c(minRange,maxRange,r2,slope,int)))){
			#if(Ct > minRange & Ct < maxRange & r2 > 0.2){
				conc <- 10^((Ct - int) / slope)
			#} else {
			#	conc <- NA	
			#}
		} else { conc <- NA}
		conc
   }
   
   
   
   
# across a particular experiment, plot the controls as a consistency check
   # for a timecourse
   plotCNTLs <- function(x,CNTLs,exptName){
   			# troubleshooting
			#x <- p1
			#exptName <- kcl_0136_p1b1
   	 		# create an average control profile
   	   		x <- x[which(!is.na(match(x$primer,CNTLs))),]
   	   		
   			x$myX <- NA
   			for(i in 1:dim(exptName)[1]){
   			   x$myX[grep(exptName[i,1],x$sampleName)] <- exptName[i,2]
   			}

   			use <- which(!is.na(x$myX))

   			means <- tapply(as.numeric(x$realConc[use]),factor(x$myX[use]),mean,na.rm=T)
		
   			#plot(x$myX[use],x$realConc[use],xlab="Time (min)",ylab="Expression",frame.plot=F,main="control genes")
   			plot(names(means),means,xlab="Time (min)",ylab="Expression",frame.plot=F,main="control genes",ylim=c(0,1.5*max(means)))
   			means
   }
   # for a timecourse with muultiple conditions
   plotCNTLsMulti <- function(x,CNTLs,exptName){
   
   			# troubleshooting

   	 		# create an average control profile
   	   		x <- x[which(!is.na(match(x$primer,CNTLs))),]

			myColors <- unique(exptName[,3])
			myLabels <- unique(exptName[,4])  	   		
   			x$myX <- NA
   			x$myColor <- NA
   			for(i in 1:dim(exptName)[1]){
   			   x$myX[which(exptName[i,1]==x$sampleName)] <- exptName[i,2]
   			   x$myColor[which(exptName[i,1]==x$sampleName)] <- exptName[i,3]
   			}

   			use <- which(!is.na(x$myX))

   			means <- by(as.numeric(x$realConc[use]),list(factor(x$myX[use]),factor(x$myColor[use])),mean,na.rm=T)[,]
		
   			if(length(dim(means)) > 1){
   			   plot(rownames(means),means[,1],xlab="Time (min)",ylab="Expression",frame.plot=F,main="control genes",ylim=c(0,1.5*max(means)))
   			   for(i in (match(myColors,colnames(means)))){
   			      points(rownames(means),means[,i],col=myColors[i])
   			   }
   			} else {
   			   names(means) <- myLabels[match(myColors,names(means))]
   			   barplot(means,xlab="Time (min)",ylab="Expression",main="control genes",ylim=c(0,1.5*max(means)))
				
   			}

   			means
   }   
   
# using the control average to normalize, plot the signal for a particular primer across the experiment
      plotPulse <- function(x,primerName,exptName,c,replot=F,myColor="black",maxY=NA,lines=NULL){


			#troubleshooting
			#c <- cntls_kcl_0136_p1b1
			#exptName <- kcl_0136_p1b1
			#primerName <- "Fosl1"
			#x <- p1
			
			#x <- p2
			#primerName="Fos"
			#exptName <- kcl_5
			#c=cntls_kcl_5

   	   		# limit x to only the relevant rows; then set the x-values for plotting (designed for timecourses)
   			x <- x[which(x$primer==primerName),]
   			x$myX <- NA
   			for(i in 1:dim(exptName)[1]){
   			   x$myX[which(exptName[i,1]==x$sampleName)] <- exptName[i,2]
   			}
   			
   			if(any(duplicated(paste(x$primer,x$sampleName)))){noTechRep <- F
   			} else {noTechRep <- TRUE}

			# across whatever replicates are available, calculate means and stderrs (normalized to the controls that were input)
   			stderr <- function(x){sd(x,na.rm=TRUE)/sqrt(length(x))}
   			use <- which(!is.na(x$myX))
   			means <- tapply(x$realConc[use],factor(x$myX[use]),mean,na.rm=T)
   			stderrs <- tapply(x$realConc[use],factor(x$myX[use]),stderr)

   			means <- means/c
   			stderrs <- stderrs / c
   			stderrs <- stderrs / means[1]   	
   			means <- means/means[1]

			if(noTechRep == T){stderrs[1:length(stderrs)] <- 0} # there are no tech reps and therefore no std errs

		    if(any(!is.nan(means) & means != Inf & means != -Inf)){ # if we can plot
		       if(replot==TRUE){
		          points(names(means),means,col=myColor,type="b")
		       } else {
		          if(is.na(maxY)){
		          		maxY <- max(means+stderrs,na.rm=TRUE)
		          		if(maxY == -Inf | maxY == Inf){maxY <- 100}
		          	}
   			      plot(names(means),means,xlab="Time (min)",ylab="Expression (AU)",frame.plot=F,main=primerName,ylim=c(0,maxY),col=myColor,type="b")
   			      if(!is.null(lines)){
   			         for(i in 1:length(lines)){abline(v=lines[i],col="green")}
   			      }
 			   }
 			
 			
			   # plot standard errors
   			   tops <- means + stderrs
   			   bots <- means - stderrs
   			   plotWidth <- diff(c(0,max(as.numeric(names(means)),na.rm=TRUE)))
   			   barWidth <- plotWidth/100
   			   for(i in 1:length(names(means))){
   			      lines(x=c(names(means)[i],names(means)[i]),y=c(tops[i],bots[i]),col=myColor) # the vertical lines
   			      lines(x=c(as.numeric(names(means)[i])-barWidth/2,as.numeric(names(means)[i])+barWidth/2),y=c(tops[i],tops[i]),col=myColor) # top caps
   			      lines(x=c(as.numeric(names(means)[i])-barWidth/2,as.numeric(names(means)[i])+barWidth/2),y=c(bots[i],bots[i]),col=myColor) # bot caps
   			   }
   			} else {
   			   plot(1:3,1:3,frame.plot=F,main=primerName,type='n',ylab="Expression (AU)",xlab="Time (min)")	
   			}
   			
   			means
   } 
   
      

# plot all primers for a given experiment, across several PDFs each filling a page
plotAllPrimers <- function(x,expt,exptCntl,primers,fN,newWindow=TRUE,lines=NULL){
   if(newWindow == TRUE){quartz(title="",width=10,height=7.5)}
   white()
   par(mar=c(4,4.5,4,1)) # bot,left,top,right
   layout(rbind(1:6,7:12))
   for(i in 1:12){
      plotPulse(x,primerName=primers[i],expt,c=exptCntl,lines=lines)
   }
   quartz.save(file=paste(fN,"_1-12.pdf",sep=""),type="pdf")
   layout(rbind(1:6,7:12))
   for(i in 13:24){
      plotPulse(x,primerName=primers[i],expt,c=exptCntl,lines=lines)
   }
   quartz.save(file=paste(fN,"_13-24.pdf",sep=""),type="pdf")
   layout(rbind(1:6,7:12))
   for(i in 25:36){
      plotPulse(x,primerName=primers[i],expt,c=exptCntl,lines=lines)
   }
   quartz.save(file=paste(fN,"_25-36.pdf",sep=""),type="pdf")
   layout(rbind(1:6,7:12))
   for(i in 37:length(primers)){   
      plotPulse(x,primerName=primers[i],expt,c=exptCntl,lines=lines)
   }
   quartz.save(file=paste(fN,"_37-47.pdf",sep=""),type="pdf")
   
   
}
   
   
   
   
   
   # using the control average to normalize, plot the signal for a particular primer across the experiment
      plotPulseMultiColor <- function(x,primerName,exptName,c,replot=F,maxY=NA,lines=NULL,singularNormalization=F){

			

			#x <- c3
			#primerName="Fos"
			#exptName= sustained_kcl_chx_ans_b1
			#c=cntls_sustained_kcl_chx_ans_b1_14
			#replot=F

			myColors <- unique(exptName[,3])
			myLabels <- unique(exptName[,4])

   	   		# limit x to only the relevant rows; then set the x-values for plotting (designed for timecourses)
   			x <- x[which(x$primer==primerName),]
   			x$myX <- NA
   			x$myColor <- NA
   			for(i in 1:dim(exptName)[1]){
   			   x$myX[which(exptName[i,1]==x$sampleName)] <- exptName[i,2]
   			   x$myColor[which(exptName[i,1]==x$sampleName)] <- exptName[i,3]
   			}
   			   			
   			if(any(duplicated(paste(x$primer,x$sampleName)))){noTechRep <- F
   			} else {noTechRep <- TRUE}

			# across whatever replicates are available, calculate means and stderrs (normalized to the controls that were input)
   			stderr <- function(x){sd(x,na.rm=TRUE)/sqrt(length(x)-1)}
   			use <- which(!is.na(x$myX))
   			myFactors <- list(factor(x$myX[use]),factor(x$myColor[use]))
   			means <- by(x$realConc[use],myFactors,mean,na.rm=T)
   			stderrs <- by(x$realConc[use],myFactors,stderr)
 
   		    ref <- means[1,1] / c[1,1]  	  
            for(i in match(myColors,colnames(means[,]))){
   			   means[,i] <- means[,i]/ c[,i]
   			   stderrs[,i] <- stderrs[,i] / c[,i] 
   			   if(singularNormalization == TRUE){
   			      stderrs[,i] <- stderrs[,i] / ref
   			      means[,i] <- means[,i]/ref
   			   } else {
				  ref <- means[1,i]
   			      stderrs[,i] <- stderrs[,i] / ref   
   			      means[,i] <- means[,i] / ref
   			   }
			   if(noTechRep == T){stderrs[1:length(stderrs[,i]),i] <- 0} # there are no tech reps and therefore no std errs
			 }

		    if(any(!is.nan(means[,]) & means[,] != Inf & means[,] != -Inf)){ # if we can plot
		       if(replot==TRUE){
	              for(i in match(myColors,colnames(means[,]))){
		             points(names(means[,i]),means[,i],col=myColors[i],type="b")
		          }
		       } else {
		          if(is.na(maxY)){
		          		maxY <- 1.2*max(means[,]+stderrs[,],na.rm=TRUE)
		          		if(maxY == -Inf | maxY == Inf){maxY <- 100}
		          }
   			      plot(rownames(means),means[,match(myColors,colnames(means[,]))[1]],xlab="Time (min)",ylab="Expression (AU)",frame.plot=F,main=primerName,ylim=c(0,maxY),col=match(myColors,colnames(means[,]))[1],type="b")
   			      for(i in match(myColors,colnames(means[,]))[2:length(match(myColors,colnames(means[,])))]){
   			      	points(names(means[,i]),means[,i],col=myColors[i],type="b")
   			      }
 			   }
 			
 			
			   # plot standard errors
   			   tops <- means[,] + stderrs[,]
   			   bots <- means[,] - stderrs[,]
   			   plotWidth <- diff(c(0,max(as.numeric(rownames(means)),na.rm=TRUE)))
   			   barWidth <- plotWidth/100
   			   for(i in 1:length(rownames(means))){
   			   	  for(j in match(myColors,colnames(means[,]))){
   			         lines(x=c(rownames(means)[i],rownames(means)[i]),y=c(tops[i,j],bots[i,j]),col=myColors[j]) # the vertical lines
   			         lines(x=c(as.numeric(rownames(means)[i])-barWidth/2,as.numeric(rownames(means)[i])+barWidth/2),y=c(tops[i,j],tops[i,j]),col=myColors[j]) # top caps
   			         lines(x=c(as.numeric(rownames(means)[i])-barWidth/2,as.numeric(rownames(means)[i])+barWidth/2),y=c(bots[i,j],bots[i,j]),col=myColors[j]) # bot caps
   			      }
   			   }
   			   legend("topright",text.col=myColors,legend=myLabels,bty='n')
   			} else {
   			   plot(1:3,1:3,frame.plot=F,main=primerName,type='n',ylab="Expression (AU)",xlab="Time (min)")	
   			}
   			
   			results <- means[,]
   			colnames(results) <- myLabels[match(myColors,colnames(means[,]))]
   			results
   } 
   
   
   
   

   # plot all primers for a given experiment, across several PDFs each filling a page
plotAllPrimersMulti <- function(x,expt,exptCntl,primers,fN,newWindow=TRUE,lines=NULL,singularNormalization=F){
   if(newWindow == TRUE){quartz(title="",width=10,height=7.5)}
   white()
   par(mar=c(4,4.5,4,1)) # bot,left,top,right
   layout(rbind(1:6,7:12))
   for(i in 1:12){
      plotPulseMultiColor(x,primerName=primers[i],expt,c=exptCntl,lines=lines,singularNormalization=singularNormalization)
   }
   quartz.save(file=paste(fN,"_1-12.pdf",sep=""),type="pdf")
   layout(rbind(1:6,7:12))
   for(i in 13:24){
      plotPulseMultiColor(x,primerName=primers[i],expt,c=exptCntl,lines=lines,singularNormalization=singularNormalization)
   }
   quartz.save(file=paste(fN,"_13-24.pdf",sep=""),type="pdf")
   layout(rbind(1:6,7:12))
   for(i in 25:36){
      plotPulseMultiColor(x,primerName=primers[i],expt,c=exptCntl,lines=lines,singularNormalization=singularNormalization)
   }
   quartz.save(file=paste(fN,"_25-36.pdf",sep=""),type="pdf")
   layout(rbind(1:6,7:12))
   for(i in 37:length(primers)){
      plotPulseMultiColor(x,primerName=primers[i],expt,c=exptCntl,lines=lines,singularNormalization=singularNormalization)
   }
   quartz.save(file=paste(fN,"_37-47.pdf",sep=""),type="pdf")
}
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
      # using the control average to normalize, plot the signal for a particular primer across the experiment
      plotBars <- function(x,primerName,exptName,c,replot=F,maxY=NA,lines=NULL){

			

			#x <- p4_14
			#primerName="Fos"
			#exptName <- zero_hrs_b1
			#c = cntls_zero_hrs_b1_14
			#exptName= sustained_kcl_chx_ans_b2
			#c=cntls_sustained_kcl_chx_ans_b2
			#myColors <- c("black","red","green")
			#replot=F

			myColors <- unique(exptName[,3])
			myLabels <- unique(exptName[,4])

   	   		# limit x to only the relevant rows; then set the x-values for plotting (designed for timecourses)
   			x <- x[which(x$primer==primerName),]
   			x$myX <- NA
   			x$myColor <- NA
   			for(i in 1:dim(exptName)[1]){
   			   x$myX[which(exptName[i,1]==x$sampleName)] <- exptName[i,2]
   			   x$myColor[which(exptName[i,1]==x$sampleName)] <- exptName[i,3]
   			}
   			   			
   			if(any(duplicated(paste(x$primer,x$sampleName)))){noTechRep <- F
   			} else {noTechRep <- TRUE}

			# across whatever replicates are available, calculate means and stderrs (normalized to the controls that were input)
   			stderr <- function(x){sd(x,na.rm=TRUE)/sqrt(length(x)-1)}
   			use <- which(!is.na(x$myX))
   			means <- tapply(x$realConc[use],factor(x$myColor[use]),mean,na.rm=T)
   			stderrs <- tapply(x$realConc[use],factor(x$myColor[use]),stderr)

			names(means) <- myLabels[match(names(means),myColors)]

   			means <- means/ c
   			stderrs <- stderrs / c 
   			stderrs <- stderrs / means[1]   
   			means <- means/means[1]
			if(noTechRep == T){stderrs[1:length(stderrs)] <- 0} # there are no tech reps and therefore no std errs
			

			

		    if(any(!is.nan(means) & means != Inf & means != -Inf)){ # if we can plot
		    	
		       barplot(height=means)	
 			
   			} else {
   			   plot(1:3,1:3,frame.plot=F,main=primerName,type='n',ylab="Expression (AU)",xlab="Time (min)")	
   			}
			means
   } 
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   