# Set your working directory to your bootcamp directory, which will differ from what's below (CHANGE THIS TO THE CORRECT DIRECTORY)
   setwd("~jmgray/Desktop/Genetics_bootcamp/")
   
# Set some default plotting parameters to make plots look nicer
   white <- function(){
	  par(bg="white",fg="black",col.lab="black",col.axis="black",col.main="black", col.sub="black")
	  par(mar=c(5.2,5.8,4,2)+0.1,cex=1,cex.main=2,cex.lab=2,cex.axis=1.5,mfrow=c(1,1),pch=19)
	  #par(xaxs='i',yaxs='i')
   }
   white()

# Import NanoString data into a data.frame called x
# There are two experiments in this dataset
# Experiment 1: Similar to the RNA-Seq experiment (sustained KCl treatment of primary cortical neurons for 0,1,3, or 6 hours)
# Experiment 2: A five-minute pulse of KCl, with a timecourse over the next 6 hours
   # Read the data file
   x <- read.delim(file="NanoString/NanoString_KCl.tsv")
   # Give the columns better names
   colnames(x)[4:15] <- c("0h_KCl","1h_KCl","3h_KCl","6h_KCl","0m_w388ngRNA","0m_post_5m_KCl_pulse","20m_post_5m_KCl_pulse","40m_post_5m_KCl_pulse","60m_post_5m_KCl_pulse","120m_post_5m_KCl_pulse","240m_post_5m_KCl_pulse","360m_post_5m_KCl_pulse")
   # Remove NanoString internal control genes for simplicity.
   x <- x[1:48,] 
   # Look at the first few rows of the data.
   head(x)

# Define a function to plot NanoString data
   # arguments
   # x -- a data.frame of imported NanoString data
   # gene -- a gene of interest to plot
   # cols -- the relevant columns of data within x
   # labs -- the numeric timepoints corresponding 
   plotNano <- function(x,gene,cols,labs,c=1,xlab="Time (min)",ylab="Counts",plotFold=F,normToCntlGenes=F){
      x <- x[which(x$Gene == gene),cols]
      if(normToCntlGenes == T){ # normalize to a set of control genes, timepoint-by-timepoint
         x <- x/c 
      }
      if(plotFold == TRUE){ # normalize to the control timepoint to plot fold induction
   	     x <- as.numeric(x)/as.numeric(x[1])
   	     ylab <- "Fold Induction (NanoString)"
      }
      plot(labs,x,frame.plot=F,xlab=xlab,ylab=ylab,type="b",ylim=c(0,max(x)+0.2*max(x)),main=gene)
   }

# Define a set of experimental genes (expected to increase with KCl treatment) and a set of control genes
# Each of these genes was designed into the NanoString CodeSet for these experiments
   # Activity-regulated genes defined from the RNA-Seq experiment
   ARG_Genes_NanoString <- c("Fosb","Npas4","Nr4a1","Junb","Fos","Egr2","Egr4","Pcsk1","Egr1","Gadd45b","Maff","Ier2","Ccnu","Klf4","Vgf","Arc","Atf7ip2","Nr4a2","Nr4a3","Egr3","Col10a1","Gprc5a","Bves","Pax1","Bdnf","Pdlim3","Acan","Npffr2","Crh","Rasd1","Serpinb2","Ankrd56","Corin","Pthlh","Fosl2")
   # Endogenous control genes chosen because they do not change with KCl in the RNA-Seq experiment
   CNTL_Genes_NanoString <- c("Opcml","Prkcn","Ncoa6","Eif5b","Hs2st1","Rab7","Rala","Pms1","AI597468","Map2k4","Zbtb1","Glra2","Gapdh")
   # The full set of primers:
   primerSet <- c(ARG_Genes_NanoString,CNTL_Genes_NanoString)
   
# Determine the average control gene signal across each timecourse (this will be useful later)
   # Control genes averaged for each timepoint of Experiment 1 (the 0, 1, 3, 6 hour sustained KCl experiment)
   (CNTLs_0136 <- round(apply(x[match(CNTLs,x$Gene),4:7],2,mean)))
   # Control genes averaged for each timepoint of Experiment 2 (the six hour timecourse following 5 minutes of KCl treatment)
   (CNTLs_5m_pulse <- round(apply(x[match(CNTLs,x$Gene),9:15],2,mean)))

# Here's the raw counts data for the 0, 1, 3, 6 hours of KCl experiment that is identical to the RNA-Seq experiment you analyzed
   x[,2:7]

# Here are three ways to plot a timecourse for a particular gene
   # Display raw counts
   plotNano(x,gene="Egr1",cols=4:7,labs=c(0,1,3,6),c=CNTLs_0136,xlab="Time (min post-KCl)") 
   # Display fold-induction
   plotNano(x,gene="Egr1",cols=4:7,labs=c(0,1,3,6),c=CNTLs_0136,xlab="Time (min post-KCl)",plotFold=T) 
   # Display fold-induction, normalized to an average of the full set of control genes listed above
   # How do you know if this is helping?
   plotNano(x,gene="Egr1",cols=4:7,labs=c(0,1,3,6),c=CNTLs_0136,xlab="Time (min post-KCl)",plotFold=T,normToCntlGenes=T) 

# Define a function to plot all genes for a given experiment, saving several PDFs each filling a full printed page
# Automatically saves PDFs in your Plot subdirectory 
   plotNanoStringAllPrimers <- function(x,expt,exptCntl,primers,fN,cols,newWindow=TRUE,lines=NULL,xlab="Time (min)",plotFold=F, normToCntlGenes=T){
      if(newWindow == TRUE){quartz(title="",width=10,height=7.5)}
      white()
      par(mar=c(4,4.5,4,1)) # bot,left,top,right
      layout(rbind(1:6,7:12))

      for(i in 1:12){
         plotNano(x,primers[i],cols,labs=expt,c=exptCntl,xlab=xlab,plotFold=plotFold,normToCntlGenes=normToCntlGenes)
      }
      quartz.save(file=paste(fN,"_1-12.pdf",sep=""),type="pdf")
      layout(rbind(1:6,7:12))
      for(i in 13:24){
         plotNano(x,primers[i],cols,labs=expt,c=exptCntl,xlab=xlab,plotFold=plotFold,normToCntlGenes=normToCntlGenes)
      }
      quartz.save(file=paste(fN,"_13-24.pdf",sep=""),type="pdf")
      layout(rbind(1:6,7:12))
      for(i in 25:36){
         plotNano(x,primers[i],cols,labs=expt,c=exptCntl,xlab=xlab,plotFold=plotFold,normToCntlGenes=normToCntlGenes)
      }
      quartz.save(file=paste(fN,"_25-36.pdf",sep=""),type="pdf")
      layout(rbind(1:6,7:12))
      for(i in 37:48){
         plotNano(x,primers[i],cols,labs=expt,c=exptCntl,xlab=xlab,plotFold=plotFold,normToCntlGenes=normToCntlGenes)
      }
      quartz.save(file=paste(fN,"_37-47.pdf",sep=""),type="pdf")
}

# Plot all genes for Experiment 1 (sustained KCl):
   # For all genes, plot normalized counts (counts / control gene counts)
   plotNanoStringAllPrimers(x,expt=c(0,1,3,6),CNTLs_0136,primers=primerSet,fN="Plots/nanostring_sustained_KCl_normalized_counts",cols=4:7,newWindow=T,xlab="Time (hours)")
   # Fold induction
   plotNanoStringAllPrimers(x,expt=c(0,1,3,6),CNTLs_0136,primers=primerSet,fN="Plots/nanostring_sustained_KCl_normalized_fold_induction",cols=4:7,newWindow=F,xlab="Time (hours)",plotFold=T)
   # un-normalized
   plotNanoStringAllPrimers(x,expt=c(0,1,3,6),CNTLs_0136,primers=primerSet,fN="Plots/nanostring_sustained_KCl_raw_counts",cols=4:7,newWindow=F,xlab="Time (hours)",plotFold=F, normToCntlGenes=F)
   
# Plot all genes for Experiment 2 (5 minute pulse of KCl):
   # Normalized counts
   plotNanoStringAllPrimers(x,expt=c(0,20,40,60,120,240,360),exptCntl= CNTLs_5m_pulse,primers=primerSet,fN="Plots/nanostring_5min_KCl_pulse_normalized_counts",cols=9:15,newWindow=T,xlab="Time (hours)")
   # fold induction
   plotNanoStringAllPrimers(x,expt=c(0,20,40,60,120,240,360),exptCntl= CNTLs_5m_pulse,primers=primerSet,fN="Plots/nanostring_5min_KCl_pulse_normalized_fold_induction",cols=9:15,newWindow=F,xlab="Time (hours)",plotFold=T)
   plotNanoStringAllPrimers(x,expt=c(0,20,40,60,120,240,360),exptCntl= CNTLs_5m_pulse,primers=primerSet,fN="Plots/nanostring_5min_KCl_pulse_raw_counts",cols=9:15,newWindow=F,xlab="Time (hours)",plotFold=F,normToCntlGenes=F)
















